//---------------------------------------------------------------------------
// JoySwitch v0.2 ALPHA, Joystick.h
// Copyright (C) 2003 by Donovan Hawkins <hawkins@cephira.com>
//
// This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//---------------------------------------------------------------------------

#include <vector>        // vector
#include <string>        // string
#include <objbase.h>     // CoInitializeEx, CoUninitialize, FAILED macro
#include "dinput.h"      // DirectInputCreate, LPDIRECTINPUT
#include "dinputd.h"     // DirectInputCreate, LPDIRECTINPUT (had to replace PHKEY with ::PHKEY for Builder)

//#include "main.h"


// Include guard

#ifndef JOYSTICK_H
#define JOYSTICK_H


// Joystick switching class (should only create one if only one instance of DirectInput is appropriate)
//  COM, IDirectInput, and IDirectInputJoyConfig are all initialized and released by this class during construction/destruction
//  Note that this class maintains exclusive access to IDirectInputJoyConfig during its lifetime to prevent changes from invalidating operator[] strings

class  TJoySwitch
{

 public:

  struct  TJoystick  // used to describe one joystick
  {
    bool         Found;   // true iff this joystick was uniquely found and can be swapped
    std::string  Name;    // joystick instance name (meaningless unless Found)
    GUID         ID;      // instance GUID
    TJoystick() : Found( false ), Name( "" ) {}
  };

  enum EError { errNone, errNoInit, errMemory, errWinVersion, errCOMInit, errDInputInit, errDInputJoyConfigInit,
                errCoopLevel, errAcquire, errEnumeration, errNoEmpty, errGetConfig, errSetConfig, errDeleteConfig };

  TJoySwitch( HINSTANCE  AppInstance, HWND  MainWindow );          // use HInstance and Application->Handle for Borland C++ Builder
  bool  Initialize( unsigned  MaxJoystickCount, DWORD  Timeout );  // Timeout in ms, returns true iff success
  ~TJoySwitch();                                                   // releases everything acquired

  EError     GetError();                  // value can be read after initialize or Swap
  TJoystick  operator[]( unsigned  ID );  // returns the corresponding TJoystick struct (ID between 0 and MaxJoystickCount-1, inclusive)

  bool  Swap( unsigned  SourceID, unsigned  DestID );  // swaps two joysticks, returns true iff success
//  bool  Rename( unsigned  ID, std::string  NewName );  // changes joystick instance name, returns true iff success XXX!!!NOT IMPLEMENTED YET!!!XXX

 protected:

  void  initialize();                   // attempts to initialize everything not yet initialized

  HINSTANCE                 hinstance;  // This application's instance handle
  HWND                      hwnd;       // This application's main window

  bool                      is_nt;      // true iff this is an NT-based OS
  LPDIRECTINPUT             di;
  LPDIRECTINPUTJOYCONFIG    dic;

  EError                    error;      // last error

  friend BOOL CALLBACK  JoystickCallback( const DIDEVICEINSTANCE*  dinst, void*  joy );
  std::vector< TJoystick >  joystick;
  int                       safeid;     // id that is safe for swapping

};


// Close the include guard

#endif
