//---------------------------------------------------------------------------
// JoySwitch v0.2 ALPHA
// Copyright (C) 2003 by Donovan Hawkins <hawkins@cephira.com>
//
// This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include <vector>        // vector
#include <utility>       // pair
#include <objbase.h>     // CoInitializeEx, CoUninitialize, FAILED macro
#include "dinput.h"      // DirectInputCreate, LPDIRECTINPUT
#include "dinputd.h"     // DirectInputCreate, LPDIRECTINPUT
#include <winerror.h>    // E_FAIL
#include <registry.hpp>  // TRegistry

#include "Joystick.h"

#include "main.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TMainForm *MainForm;
//---------------------------------------------------------------------------
#undef  JOYTEST
//---------------------------------------------------------------------------
TJoySwitch  joyswitch( HInstance, Application->Handle );
unsigned  CommandJoystickCount = 16;     // default max joystick count
unsigned  CommandTimeout       = 0;      // default timeout delay in ms
bool      CommandVerbose       = false;  // default setting for showing error messageboxes (unless GUI)
std::vector< std::pair< AnsiString, AnsiString > >  CommandSwap;
//---------------------------------------------------------------------------
// Function to update the page
void  JoystickUpdate()
{
  MainForm->JoystickRadioGroup->Items->Clear();
  for ( unsigned  i = 0; i < CommandJoystickCount; ++i )
  {
    if ( joyswitch[ i ].Found )
    {
      AnsiString  s = AnsiString( i ) + ": " + AnsiString( joyswitch[ i ].Name.c_str() );
      MainForm->JoystickRadioGroup->Items->Add( s.c_str() );
    }
  }
  MainForm->JoystickRadioGroup->ItemIndex = ( joyswitch[ 0 ].Found ? 0 : -1 );
  return;
}
//---------------------------------------------------------------------------
// Function to handle command-line mode, returns true iff success
bool  JoystickCommand()
{
  CommandSwap.clear();
  for ( int  i = 1; i <= ParamCount(); ++i )
  {
    int  j;
    AnsiString  s = ParamStr( i );
    if ( s.Length() < 1 )  return  false;  // shouldn't happen
    if ( ( s[ 1 ] == '/' ) || ( s[ 1 ] == '-' ) )
    {
      if ( s.Length() < 2 )  return  false;  // no switch letter
      switch  ( s[ 2 ] )
      {
        case 'm':
        case 'M':
          if ( s.Length() < 3 )  return  false;  // no value
          j = s.SubString( 3, s.Length() - 2 ).ToIntDef( -1 );
          if ( j < 1 )  return  false;  // invalid value
          CommandJoystickCount = j;
          break;
        case 't':
        case 'T':
          if ( s.Length() < 3 )  return  false;  // no value
          j = s.SubString( 3, s.Length() - 2 ).ToIntDef( -1 );
          if ( j < 0 )  return  false;  // invalid value
          CommandTimeout = j;
          break;
        case 'v':
        case 'V':
          if ( s.Length() > 2 )  return  false;  // no value wanted
          CommandVerbose = true;
          break;
        default:
          return  false;  // invalid command switch
          break;
      }
    }
    else
    {
      if ( s.Length() < 11 )  return  false;  // looking for ID(N),ID(N) at least
      if ( s.SubString( 1, 3 ).UpperCase() != "ID(" )  return  false;
      j = s.Pos(")");
      if ( !j )  return  false;  // no closing parentheses
      AnsiString  source = s.SubString( 4, j - 4 );
      s = s.SubString( source.Length() + 5, s.Length() - source.Length() - 4 );
      if ( s.Length() < 6 )  return  false;  // looking for ,ID(N) at least
      if ( s.SubString( 1, 4 ).UpperCase() != ",ID(" )  return  false;
      j = s.Pos(")");
      if ( !j )  return  false;  // no closing parentheses
      CommandSwap.push_back( std::make_pair( source, s.SubString( 5, j - 5 ) ) );
    }
  }
#ifdef  JOYTEST
  AnsiString  s = "Params:\nMaxJoystickCount = " + AnsiString( CommandJoystickCount ) + "\nTimeout = "
    + AnsiString( CommandTimeout ) + "\nVerbose " + AnsiString( CommandVerbose ? "On" : "Off" )
    + "\nSwap List:";
  for ( unsigned  i = 0; i < CommandSwap.size(); ++i )
  {
    s = s + "\n" + CommandSwap[ i ].first + ", " + CommandSwap[ i ].second;
  }
  Application->MessageBox( s.c_str(), "Test", MB_OK );
#endif
  return  true;
}
//---------------------------------------------------------------------------
int  GetID( AnsiString  param )
{
  int  j = param.ToIntDef( -1 );
  if ( j >= 0 )  return  j;  // a number was selected

 // case-sensitive search
  for ( unsigned  i = 0; i < CommandJoystickCount; ++i )
  {
    AnsiString  s = AnsiString( joyswitch[ i ].Name.c_str() );
    if ( s == param )  return  i;  // return first match
  }

 // case-insensitive search
  for ( unsigned  i = 0; i < CommandJoystickCount; ++i )
  {
    AnsiString  s = AnsiString( joyswitch[ i ].Name.c_str() );
    if ( s.UpperCase() == param.UpperCase() )  return  i;  // return first match
  }

 // partial case-sensitive search
  for ( unsigned  i = 0; i < CommandJoystickCount; ++i )
  {
    AnsiString  s = AnsiString( joyswitch[ i ].Name.c_str() );
    s.SetLength( param.Length() );
    if ( s == param )  return  i;  // return first match
  }

 // partial case-insensitive search
  for ( unsigned  i = 0; i < CommandJoystickCount; ++i )
  {
    AnsiString  s = AnsiString( joyswitch[ i ].Name.c_str() );
    s.SetLength( param.Length() );
    if ( s.UpperCase() == param.UpperCase() )  return  i;  // return first match
  }

  return  -1;  // not found
}
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
        : TForm(Owner)
{
 // Get command parameters, if any
  if ( ParamCount() )
  {
    if ( !JoystickCommand() )
    {
      ExitCode = -1;  // Syntax error in JoystickCommand
      Application->Terminate();
      return;
    }
  }
  else  CommandVerbose = true;  // GUI mode is always verbose

 // Initialize
  if ( !joyswitch.Initialize( CommandJoystickCount, CommandTimeout ) )
  {
    ExitCode = joyswitch.GetError();
    AnsiString  s = "Error code is " + AnsiString( joyswitch.GetError() );
    if ( CommandVerbose )  Application->MessageBox( s.c_str(), "Initialization Error", MB_OK );
    Application->Terminate();
    return;
  }

 // Either run commands or run GUI
  if ( ParamCount() )
  {
    if ( CommandSwap.size() == 0 )  // nothing to do
    {
      if ( CommandVerbose )  Application->MessageBox( "Nothing to do", "Command Warning", MB_OK );
      ExitCode = -2;  // No swap lists found
      Application->Terminate();
      return;
    }
    for ( unsigned  i = 0; i < CommandSwap.size(); ++i )
    {
      int  source = GetID( CommandSwap[ i ].first );
      int  dest   = GetID( CommandSwap[ i ].second );
      if ( ( source < 0 ) || ( dest < 0 ) || ( static_cast< unsigned >( source ) >= CommandJoystickCount )
                                          || ( static_cast< unsigned >( dest   ) >= CommandJoystickCount ) )
      {
        if ( CommandVerbose )  Application->MessageBox( "Nothing to do", "Command Warning", MB_OK );
        ExitCode = -3;  // Joysticks not found
        Application->Terminate();
        return;
      }
     #ifdef  JOYTEST
      AnsiString  s = "Swap " + AnsiString( source ) + ", " + AnsiString( dest );
      Application->MessageBox( s.c_str(), "Test", MB_OK );
     #else
       joyswitch.Swap( source, dest );
     #endif
    }
    ExitCode = 0;
    Application->Terminate();
    return;
  }
  else
  {
    ExitCode = 0;
    JoystickUpdate();
    MainForm->Show();
  }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::RefreshButtonClick(TObject *Sender)
{
  JoystickUpdate();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::DoneButtonClick(TObject *Sender)
{
  if ( JoystickRadioGroup->ItemIndex < 0 )                       MainForm->Close();
  if ( joyswitch[ 0 ].Found && !JoystickRadioGroup->ItemIndex )  MainForm->Close();
  unsigned  i = JoystickRadioGroup->Items->Strings[ JoystickRadioGroup->ItemIndex ].Pos( ":" );
  if ( i )  i = JoystickRadioGroup->Items->Strings[ JoystickRadioGroup->ItemIndex ].SubString( 1, i - 1 ).ToIntDef( 0 );
 #ifdef  JOYTEST
  AnsiString  s = "Swap " + AnsiString( i ) + ", " + AnsiString( 0 );
  Application->MessageBox( s.c_str(), "Test", MB_OK );
  MainForm->Close();
 #else
  if ( joyswitch.Swap( i, 0 ) )      MainForm->Close();
  else
  {
    AnsiString  s = "Error code is " + AnsiString( joyswitch.GetError() );
    Application->MessageBox( s.c_str(), "Error", MB_OK );
  }
 #endif
}
//---------------------------------------------------------------------------

